<?php

/**
 * @file
 * Handles admin settings page for Commerce Correios Local module.
 */

/**
 * Module settings form.
 */
function commerce_correios_local_settings_form($form, &$form_state, $service) {
  $services_enabled = array_filter(variable_get('commerce_correios_services', array()));
  $form['commerce_correios_local_service'] = array(
    '#type' => 'value',
    '#value' => $service,
  );
  $form['commerce_correios_local_service_' . $service] = commerce_correios_local_matrix_form($service);

  $form['actions']['#type'] = 'actions';
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configurations'),
    '#submit' => array('commerce_correios_local_save_submit'),
  );
  $form['actions']['update'] = array(
    '#type' => 'submit',
    '#value' => t('Update rates'),
    '#submit' => array('commerce_correios_local_update_rates_submit'),
    '#weight' => 10,
  );
  $form['actions']['update_failed'] = array(
    '#type' => 'submit',
    '#value' => t('Update failed rates'),
    '#submit' => array('commerce_correios_local_update_failed_rates_submit'),
    '#weight' => 15,
  );

  return $form;
}

function commerce_correios_local_matrix_form($service) {
  $services_available = _commerce_correios_services_types();

  $form['#tree'] = TRUE;
  $form['info'] = array(
    '#markup' => '<h2>' . $services_available[$service]['title'] . '</h2>'
  );
  $form['rates'] = array(
    '#theme' => 'table',
  );

  $ranges = commerce_correios_local_ranges();
  $weights = commerce_correios_local_weights();

  $form['rates']['#header'] = array_merge(array(t('Weights/States')), $weights);

  // Get values from last update for reviewing/saving or canceling rates update.
  $values = variable_get('commerce_correios_local_service_' . $service . '_temporary', array());

  if (empty($values)) {
    $values = variable_get('commerce_correios_local_service_' . $service, array());
  }

  foreach ($ranges as $state => $info) {
    foreach (array('capital', 'interior') as $kind) {
      $row_header = $state . ' ' . ($kind == 'capital' ? t('Capital') : t('Interior'));
      $row = array($row_header);
      foreach ($weights as $weight) {
        $rate_key = strtolower($state) . '_' . $kind . '_' . $weight;
        $form['rates'][$rate_key] = array(
          '#type' => 'textfield',
          '#default_value' => $values[$rate_key],
          '#element_validate' => array('element_validate_integer_positive'),
          '#size' => 5,
          '#parents' => array('commerce_correios_local_service_' . $service, $rate_key),
        );
        $row[] = array('data' => &$form['rates'][$rate_key]);
      }
      $form['rates']['#rows'][] = $row;
    }
  }

  return $form;
}

/**
 * Module settings save submit callback.
 */
function commerce_correios_local_save_submit($form, &$form_state) {
  $service = $form_state['values']['commerce_correios_local_service'];
  $values = $form_state['values']['commerce_correios_local_service_' . $service];
  variable_set('commerce_correios_local_service_' . $service, $values);
  variable_del('commerce_correios_local_service_' . $service . '_temporary');
}

/**
 * Submit callback for updating rates.
 */
function commerce_correios_local_update_rates_submit($form, &$form_state) {
  $service = $form_state['values']['commerce_correios_local_service'];
  $batch = array(
    'title' => t('Updating rates'),
    'operations' => array(
       array('commerce_correios_local_update_rates', array($service, FALSE)),
    ),
    'finished' => 'commerce_correios_local_update_rates_finished',
    'file' => drupal_get_path('module', 'commerce_correios_local') . '/commerce_correios_local.admin.inc',
  );

  batch_set($batch);
}

/**
 * Submit callback for updating preivously failed rates, i.e. the ones which
 * have 0 as rate.
 */
function commerce_correios_local_update_failed_rates_submit($form, &$form_state) {
  $service = $form_state['values']['commerce_correios_local_service'];
  $batch = array(
    'title' => t('Updating failed rates'),
    'operations' => array(
      array('commerce_correios_local_update_rates', array($service, TRUE)),
    ),
    'finished' => 'commerce_correios_local_update_rates_finished',
    'file' => drupal_get_path('module', 'commerce_correios_local') . '/commerce_correios_local.admin.inc',
  );

  batch_set($batch);
}

/**
 * Update rates batch operation.
 */
function commerce_correios_local_update_rates($service, $failed_only, $context) {
  $zipcodes = commerce_correios_local_reference_zipcodes();
  $states = array_keys($zipcodes);

  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_index'] = 0;
    $context['sandbox']['max'] = count($states);
    $context['results'] = array(
      'results' => array(),
      'service' => $service,
    );
  }

  // Process 3 states at a time.
  $limit = $context['sandbox']['current_index'] + 3;

  while ($context['sandbox']['current_index'] < min($limit, $context['sandbox']['max'])) {
    $state = $states[$context['sandbox']['current_index']];
    $rate = commerce_correios_local_update_rate($service, $state);
    $context['results']['results'] = array_merge($context['results']['results'], $rate);
    $context['message'] = t('Now processing state %state', array('%state' => $context['sandbox']['current_index']));

    // Update our progress information.
    $context['sandbox']['progress']++;
    $context['sandbox']['current_index']++;
  }

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * Update rate.
 */
function commerce_correios_local_update_rate($service, $state) {
  $zipcodes = commerce_correios_local_reference_zipcodes();
  $weights = commerce_correios_local_weights();
  $result = array();
  foreach ($zipcodes[$state] as $variant => $zipcode) {
    foreach ($weights as $weight) {
      $key = strtolower($state) . '_' . $variant . '_' . $weight;
      $result[$key] = _commerce_correios_local_update_rate($service, $zipcode, $weight);
    }
  }
  return $result;
}

function _commerce_correios_local_update_rate($service, $zipcode, $weight) {
  $params = array(
    'nCdServico' => $service,
    'sCepOrigem' => variable_get('commerce_correios_store_postal_code', ''),
    'sCepDestino' => $zipcode,
    'nVlPeso' => round($weight / 1000, 2),
    'nVlLargura' => variable_get('commerce_correios_default_width', 11),
    'nVlAltura' => variable_get('commerce_correios_default_height', 2),
    'nVlComprimento' => variable_get('commerce_correios_default_length', 16),
  );

  // Add contract data.
  $contract = variable_get('commerce_correios_contract_number', '');
  if (!empty($contract)) {
    $params['nCdEmpresa'] = $contract;
    $params['sDsSenha'] = variable_get('commerce_correios_password', '');
  }

  $rate = commerce_correios_call($params);

  return commerce_currency_decimal_to_amount(str_replace(',', '.', (string) $rate->Valor), 'BRL');
}

/**
 * Update rates batch finished callback.
 */
function commerce_correios_local_update_rates_finished($success, $results, $operations) {
  if ($success) {
    variable_set('commerce_correios_local_service_' . $results['service'] . '_temporary', $results['results']);
    drupal_set_message(count($results['results']) . ' rates updated.');
  }
  else {
    // An error occurred.
    // $operations contains the operations that remained unprocessed.
    $error_operation = reset($operations);
    $message = t('An error occurred while processing %error_operation with arguments: @arguments', array('%error_operation' => $error_operation[0], '@arguments' => print_r($error_operation[1], TRUE)));
    drupal_set_message($message, 'error');
  }
}
